﻿namespace Battleship;
/// <summary>
/// Класс, отвечающий за отрисовку и перемещение объекта-сущности
/// </summary>
public class DrawingBattleship
{

    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityBattleship? EntityBattleship { get; private set; }

    /// <summary>
    /// Ширина
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки линкора
    /// </summary>
    private int? _startPosX;

    /// <summary>
    /// Верхняя координата прорисовки линкора
    /// </summary>
    private int? _startPosY;

    /// <summary>
    /// Ширина прорисовки линкора
    /// </summary>
    private readonly int _drawingBattleshipWidth = 120;

    /// <summary>
    /// Высота прорисовки линкора
    /// </summary>
    private readonly int _drawingBattleshipHeight = 80;

    /// <summary>
    /// 
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="bodyDeck">Признак наличия палубы</param>
    /// <param name="compartment">Признак наличия отсека для ракет</param>
    /// <param name="tower">Признак наличия башни</param>
    public void Init(int speed, double weight, Color bodyColor, Color additionalColor, bool bodyDeck, bool compartment, bool tower)
    {
        EntityBattleship = new EntityBattleship();
        EntityBattleship.Init(speed, weight, bodyColor, additionalColor, bodyDeck, compartment, tower);
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }


    /// <summary>
    /// 
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>

    public bool SetPictureSize(int width, int height)
    {
        //TODO проверка, что объект "влезает" в размеры поля
        //если влезает, сохраняем границы и корректируем позицию объекта, если она уже была установлена
        _pictureWidth = width;
        _pictureHeight = height;
        if (_pictureWidth <= _drawingBattleshipWidth || _pictureHeight <= _drawingBattleshipHeight)
        {
            _pictureHeight = null;
            _pictureWidth = null;
            return false;
        }
        if (_startPosX + _drawingBattleshipWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth.Value - _drawingBattleshipWidth;
        }
        if (_startPosY + _drawingBattleshipHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight.Value - _drawingBattleshipHeight;
        }
        return true;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    
    public void SetPosition(int x, int y, int width, int height)
    {
        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах
        if (x >= 0 && x + _drawingBattleshipWidth <= width && y >= 0 && y + _drawingBattleshipWidth <= height)
        {
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
        }
        
    }

    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещение выполнено, false - перемещение невозможно</returns>

    public bool MoveTransport(DirectionType direction)
    {
        if (EntityBattleship == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction) 
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityBattleship.Step > 0)
                {
                    _startPosX -= (int)EntityBattleship.Step;
                }
                return true;

            //вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityBattleship.Step + _drawingBattleshipWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityBattleship.Step;
                }
                return true; 
            
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityBattleship.Step > 0)
                {
                    _startPosY -= (int)EntityBattleship.Step;
                }
                return true;

            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityBattleship.Step + _drawingBattleshipHeight < _pictureHeight) //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                {
                    _startPosY += (int)EntityBattleship.Step;
                }
                return true;

            default: 
                return false;  
        }
        
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>

    public void DrawTransport(Graphics g)
    {
        if (EntityBattleship == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);
        Brush additionalBrush = new SolidBrush(EntityBattleship.AdditionalColor);
        
        //заливка линкора
        Brush br = new SolidBrush(EntityBattleship.BodyColor);
        g.FillRectangle(br, _startPosX.Value + 10, _startPosY.Value, 80, 40);
        //границы линкора
        g.DrawRectangle(pen, _startPosX.Value + 10, _startPosY.Value, 80, 40); //ширина высота (сама палуба)

        //заливка двигателей (2 фигни сзади)
        Brush brBlack = new SolidBrush(Color.Black);
        g.FillRectangle(brBlack, _startPosX.Value + 5, _startPosY.Value + 7, 5, 10); //верхний
        g.FillRectangle(brBlack, _startPosX.Value + 5, _startPosY.Value + 22, 5, 10); //нижний
        //границы двигателей
        g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value + 7, 5, 10); //двигатель верхний
        g.DrawRectangle(pen, _startPosX.Value + 5, _startPosY.Value + 22, 5, 10); //двигатель нижний

        //прямоугольник левый на палубе (горизонтальный)
        Brush brLightSteelBlue = new SolidBrush(Color.LightSteelBlue);
        g.FillRectangle(brLightSteelBlue, _startPosX.Value + 32, _startPosY.Value + 13, 21, 12);
        g.DrawRectangle(pen, _startPosX.Value + 32, _startPosY.Value + 13, 21, 12);

        //прямоугольник правый на палубе (вертикальный)
        Brush brGray = new SolidBrush(Color.Gray);
        g.FillRectangle(brGray, _startPosX.Value + 53, _startPosY.Value + 7, 16, 24);
        g.DrawRectangle(pen, _startPosX.Value + 53, _startPosY.Value + 7, 16, 24);

        //линии для треугольника
        Point[] pointsNOS =
        {
            new Point(_startPosX.Value + 90, _startPosY.Value),
            new Point(_startPosX.Value + 120, _startPosY.Value + 20),
            new Point(_startPosX.Value + 90, _startPosY.Value + 40),
           
        };
        g.DrawPolygon(pen, pointsNOS);
        g.FillPolygon(br, pointsNOS);

        //заливка круга
        Brush brLightSlateGray = new SolidBrush(Color.LightSlateGray);
        g.FillEllipse(brLightSlateGray, _startPosX.Value + 78, _startPosY.Value + 12, 15, 15);
        //круг на палубе
        g.DrawEllipse(pen, _startPosX.Value + 78, _startPosY.Value + 12, 15, 15);

        //ПРОВЕРКА

        if (EntityBattleship.Compartment)
        {
            //отсеки для ракет
            //заливка отсеков

            g.FillRectangle(additionalBrush, _startPosX.Value + 15, _startPosY.Value + 5, 27, 5); //верхний отсек для ракет
            g.FillRectangle(additionalBrush, _startPosX.Value + 15, _startPosY.Value + 28, 27, 5); //нижний отсек для ракет

            //границы отсеков
            g.DrawRectangle(pen, _startPosX.Value + 15, _startPosY.Value + 5, 27, 5); //верхний отсек
            g.DrawRectangle(pen, _startPosX.Value + 15, _startPosY.Value + 28, 27, 5); //нижний отсек

        }

        if (EntityBattleship.Tower)
        {
            //заливка башни
            g.FillEllipse(additionalBrush, _startPosX.Value + 98, _startPosY.Value + 15, 10, 10);
            g.FillRectangle(additionalBrush, _startPosX.Value + 107, _startPosY.Value + 17, 30, 5);

            //границы башни
            g.DrawEllipse(pen, _startPosX.Value + 98, _startPosY.Value + 15, 10, 10);
            g.DrawRectangle(pen, _startPosX.Value + 107, _startPosY.Value + 17, 30, 5);
        }
    }
}
